@@ -17,7 +17,7 @@
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2019      UT-Battelle, LLC. All rights reserved.
  *
- * Copyright (c) 2021-2024 Nanook Consulting  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * Copyright (c) 2022      IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
@@ -80,6 +80,7 @@ void prte_rmaps_base_map_job(int fd, short args, void *cbdata)
     pmix_data_array_t *darray = NULL;
     pmix_list_t nodes;
     int slots, len;
+    bool flag, *fptr;
 
     PRTE_HIDE_UNUSED_PARAMS(fd, args);
 
@@ -99,6 +100,7 @@ void prte_rmaps_base_map_job(int fd, short args, void *cbdata)
     memset(&options, 0, sizeof(prte_rmaps_options_t));
     options.stream = prte_rmaps_base_framework.framework_output;
     options.verbosity = 5;  // usual value for base-level functions
+    fptr = &flag;
 
     /* check and set some general options */
     if (prte_get_attribute(&jdata->attributes, PRTE_JOB_DO_NOT_LAUNCH, NULL, PMIX_BOOL)) {
@@ -226,9 +228,9 @@ void prte_rmaps_base_map_job(int fd, short args, void *cbdata)
             /* we do allow inheritance of the defaults */
             inherit = true;
         } else if (NULL != (parent = prte_get_job_data_object(nptr->nspace))) {
-            if (prte_get_attribute(&jdata->attributes, PRTE_JOB_INHERIT, NULL, PMIX_BOOL)) {
+            if (prte_get_attribute(&parent->attributes, PRTE_JOB_INHERIT, NULL, PMIX_BOOL)) {
                 inherit = true;
-            } else if (prte_get_attribute(&jdata->attributes, PRTE_JOB_NOINHERIT, NULL, PMIX_BOOL)) {
+            } else if (prte_get_attribute(&parent->attributes, PRTE_JOB_NOINHERIT, NULL, PMIX_BOOL)) {
                 inherit = false;
                 parent = NULL;
             } else {
@@ -281,6 +283,13 @@ void prte_rmaps_base_map_job(int fd, short args, void *cbdata)
                     }
                 }
             }
+            /* if not already assigned, inherit the parent's GPU support directive */
+            if (!prte_get_attribute(&jdata->attributes, PRTE_JOB_GPU_SUPPORT, NULL, PMIX_BOOL)) {
+                if (prte_get_attribute(&parent->attributes, PRTE_JOB_GPU_SUPPORT, (void **) &fptr, PMIX_BOOL)) {
+                    prte_set_attribute(&jdata->attributes, PRTE_JOB_GPU_SUPPORT, PRTE_ATTR_GLOBAL, fptr, PMIX_BOOL);
+                }
+            }
+
         } else {
             if (!prte_get_attribute(&jdata->attributes, PRTE_JOB_HWT_CPUS, NULL, PMIX_BOOL) &&
                 !prte_get_attribute(&jdata->attributes, PRTE_JOB_CORE_CPUS, NULL, PMIX_BOOL)) {
@@ -494,15 +503,27 @@ void prte_rmaps_base_map_job(int fd, short args, void *cbdata)
                 app->num_procs = PMIX_ARGV_COUNT_COMPAT(ck);
                 PMIX_ARGV_FREE_COMPAT(ck);
             } else {
-                /* set the num_procs to equal the number of slots on these
-                 * mapped nodes, taking into account the number of cpus/rank
-                 */
-                app->num_procs = slots / options.cpus_per_rank;
-                /* sometimes, we have only one "slot" assigned, but may
-                 * want more than one cpu/rank - so ensure we always wind
-                 * up with at least one proc */
-                if (0 == app->num_procs) {
-                    app->num_procs = 1;
+                if (1 < options.cpus_per_rank) {
+                    // compute the number of cpus on each node
+                    len = 0;
+                    PMIX_LIST_FOREACH (node, &nodes, prte_node_t) {
+                        if (options.use_hwthreads) {
+                            len += prte_hwloc_base_get_nbobjs_by_type(node->topology->topo,
+                                                                      HWLOC_OBJ_PU, 0) / options.cpus_per_rank;
+                        } else {
+                            len += prte_hwloc_base_get_nbobjs_by_type(node->topology->topo,
+                                                                      HWLOC_OBJ_CORE, 0) / options.cpus_per_rank;
+                        }
+                    }
+                    app->num_procs = len;
+                    // ensure we always wind up with at least one proc
+                    if (0 == app->num_procs) {
+                        app->num_procs = 1;
+                    } else if (slots < app->num_procs) {
+                        app->num_procs = slots;
+                    }
+                } else {
+                    app->num_procs = slots;
                 }
             }
         }
