@@ -2,7 +2,7 @@
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University.
  *                         All rights reserved.
- * Copyright (c) 2004-2005 The Trustees of the University of Tennessee.
+ * Copyright (c) 2004-2017 The Trustees of the University of Tennessee.
  *                         All rights reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
@@ -20,6 +20,7 @@
 
 #include "ompi_config.h"
 
+#include "opal/util/show_help.h"
 #include "ompi/constants.h"
 #include "ompi/mca/mca.h"
 #include "opal/mca/base/base.h"
@@ -35,7 +36,6 @@ ompi_osc_base_select(ompi_win_t *win,
                      size_t size,
                      int disp_unit,
                      ompi_communicator_t *comm,
-                     opal_info_t *info,
                      int flavor,
                      int *model)
 {
@@ -54,12 +54,9 @@ ompi_osc_base_select(ompi_win_t *win,
         ompi_osc_base_component_t *component = (ompi_osc_base_component_t*)
             ((mca_base_component_list_item_t*) item)->cli_component;
 
-        priority = component->osc_query(win, base, size, disp_unit, comm, info, flavor);
+        priority = component->osc_query(win, base, size, disp_unit, comm,
+                                        win->super.s_info, flavor);
         if (priority < 0) {
-            if (MPI_WIN_FLAVOR_SHARED == flavor && OMPI_ERR_RMA_SHARED == priority) {
-                /* NTH: quick fix to return OMPI_ERR_RMA_SHARED */
-                return OMPI_ERR_RMA_SHARED;
-            }
             continue;
         }
 
@@ -71,5 +68,20 @@ ompi_osc_base_select(ompi_win_t *win,
 
     if (NULL == best_component) return OMPI_ERR_NOT_SUPPORTED;
 
-    return best_component->osc_select(win, base, size, disp_unit, comm, info, flavor, model);
+#if OPAL_ENABLE_FT_MPI
+    if(ompi_ftmpi_enabled) {
+        /* check if module is tested for FT, warn if not. */
+        const char* ft_whitelist="";
+        opal_show_help("help-mpi-ft.txt", "module:untested:failundef", true,
+            best_component->osc_version.mca_type_name,
+            best_component->osc_version.mca_component_name,
+            ft_whitelist);
+    }
+#endif /* OPAL_ENABLE_FT_MPI */
+    opal_output_verbose( 10, ompi_osc_base_framework.framework_output,
+                         "select: component %s selected",
+                         best_component->osc_version.mca_component_name );
+
+    return best_component->osc_select(win, base, size, disp_unit, comm,
+                                      win->super.s_info, flavor, model);
 }
