@@ -3,21 +3,23 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2009      Sun Microsystmes, Inc.  All rights reserved.
  * Copyright (c) 2011      Sandia National Laboratories. All rights reserved.
- * Copyright (c) 2014      Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2014-2015 Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 #include "ompi_config.h"
@@ -33,19 +35,18 @@
 #include "ompi/datatype/ompi_datatype_internal.h"
 #include "ompi/memchecker.h"
 
-#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OMPI_BUILD_MPI_PROFILING
+#if OPAL_HAVE_WEAK_SYMBOLS
 #pragma weak MPI_Raccumulate = PMPI_Raccumulate
 #endif
-
-#if OMPI_PROFILING_DEFINES
-#include "ompi/mpi/c/profile/defines.h"
+#define MPI_Raccumulate PMPI_Raccumulate
 #endif
 
 static const char FUNC_NAME[] = "MPI_Raccumulate";
 
-int MPI_Raccumulate(void *origin_addr, int origin_count, MPI_Datatype origin_datatype,
+int MPI_Raccumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype,
                    int target_rank, MPI_Aint target_disp, int target_count,
-                   MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request) 
+                   MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request)
 {
     int rc;
     ompi_win_t *ompi_win = (ompi_win_t*) win;
@@ -62,7 +63,7 @@ int MPI_Raccumulate(void *origin_addr, int origin_count, MPI_Datatype origin_dat
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
 
         if (ompi_win_invalid(win)) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_WIN, FUNC_NAME);
+            return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_WIN, FUNC_NAME);
         } else if (origin_count < 0 || target_count < 0) {
             rc = MPI_ERR_COUNT;
         } else if (ompi_win_peer_invalid(win, target_rank) &&
@@ -72,7 +73,7 @@ int MPI_Raccumulate(void *origin_addr, int origin_count, MPI_Datatype origin_dat
             rc = MPI_ERR_OP;
         } else if (!ompi_op_is_intrinsic(op)) {
             rc = MPI_ERR_OP;
-        } else if ( target_disp < 0 ) {
+        } else if ( MPI_WIN_FLAVOR_DYNAMIC != win->w_flavor && target_disp < 0 ) {
             rc = MPI_ERR_DISP;
         } else {
             OMPI_CHECK_DATATYPE_FOR_ONE_SIDED(rc, origin_datatype, origin_count);
@@ -91,8 +92,8 @@ int MPI_Raccumulate(void *origin_addr, int origin_count, MPI_Datatype origin_dat
 
                     /* RACCUMULATE, unlike REDUCE, can use with derived
                        datatypes with predefinied operations, with some
-                       restrictions outlined in MPI-2:6.3.4.  The derived
-                       datatype must be composed entierly from one predefined
+                       restrictions outlined in MPI-3:11.3.4.  The derived
+                       datatype must be composed entirely from one predefined
                        datatype (so you can do all the construction you want,
                        but at the bottom, you can only use one datatype, say,
                        MPI_INT).  If the datatype at the target isn't
@@ -126,16 +127,13 @@ int MPI_Raccumulate(void *origin_addr, int origin_count, MPI_Datatype origin_dat
         return MPI_SUCCESS;
     }
 
-    OPAL_CR_ENTER_LIBRARY();
-
-    /* TODO: don't cast away the const */
-    rc = ompi_win->w_osc_module->osc_raccumulate((void*) origin_addr,
+    rc = ompi_win->w_osc_module->osc_raccumulate(origin_addr,
                                                 origin_count,
                                                 origin_datatype,
-                                                target_rank, 
-                                                target_disp, 
+                                                target_rank,
+                                                target_disp,
                                                 target_count,
-                                                target_datatype, 
+                                                target_datatype,
                                                 op, win, request);
     OMPI_ERRHANDLER_RETURN(rc, win, rc, FUNC_NAME);
 }
