@@ -20,6 +20,7 @@
  * Copyright (c) 2014-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2018-2022 Amazon.com, Inc. or its affiliates.  All Rights reserved.
+ * Copyright (c) 2023      Jeffrey M. Squyres.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -472,7 +473,7 @@ static int mca_btl_tcp_component_close(void)
     OBJ_DESTRUCT(&mca_btl_tcp_component.tcp_frag_max);
     OBJ_DESTRUCT(&mca_btl_tcp_component.tcp_frag_user);
     OBJ_DESTRUCT(&mca_btl_tcp_component.tcp_lock);
-    OBJ_DESTRUCT(&mca_btl_tcp_component.local_ifs);
+    OPAL_LIST_DESTRUCT(&mca_btl_tcp_component.local_ifs);
 
     return OPAL_SUCCESS;
 }
@@ -667,6 +668,7 @@ static char **split_and_resolve(char **orig_str, char *name, bool reqd)
                                     "btl: tcp: Using interface: %s ", argv[i]);
                 opal_argv_append(&interface_count, &interfaces, argv[i]);
             }
+            free(argv[i]);
             continue;
         }
 
