@@ -15,8 +15,8 @@
 #include "opal_config.h"
 #include "opal/types.h"
 
-#include "opal/mca/mca.h"
 #include "opal/mca/base/mca_base_framework.h"
+#include "opal/mca/mca.h"
 
 #include "opal/mca/reachable/reachable.h"
 
@@ -29,6 +29,9 @@ OPAL_DECLSPEC extern mca_base_framework_t opal_reachable_base_framework;
  */
 OPAL_DECLSPEC int opal_reachable_base_select(void);
 
+OPAL_DECLSPEC opal_reachable_t *opal_reachable_allocate(unsigned int num_local,
+                                                        unsigned int num_remote);
+
 END_C_DECLS
 
 #endif
