@@ -3,9 +3,9 @@
  *                         All rights reserved.
  * Copyright (c) 2013 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -18,12 +18,18 @@
 #include "ompi/datatype/ompi_datatype.h"
 #include "stdio.h"
 
+#if OSHMEM_PROFILING
+#include "oshmem/shmem/fortran/pbindings.h"
+SHMEM_GENERATE_WEAK_BINDINGS(SHMEM_INTEGER_GET, shmem_integer_get)
+#include "oshmem/shmem/fortran/profile-defines.h"
+#endif
+
 SHMEM_GENERATE_FORTRAN_BINDINGS_SUB (void,
         SHMEM_INTEGER_GET,
         shmem_integer_get_,
         shmem_integer_get__,
         shmem_integer_get_f,
-        (FORTRAN_POINTER_T target, FORTRAN_POINTER_T source, MPI_Fint *len, MPI_Fint *pe), 
+        (FORTRAN_POINTER_T target, FORTRAN_POINTER_T source, MPI_Fint *len, MPI_Fint *pe),
         (target,source,len,pe) )
 
 void shmem_integer_get_f(FORTRAN_POINTER_T target, FORTRAN_POINTER_T source, MPI_Fint *len, MPI_Fint *pe)
@@ -31,9 +37,9 @@ void shmem_integer_get_f(FORTRAN_POINTER_T target, FORTRAN_POINTER_T source, MPI
     size_t integer_type_size = 0;
     ompi_datatype_type_size(&ompi_mpi_integer.dt, &integer_type_size);
 
-    MCA_SPML_CALL(get(FPTR_2_VOID_PTR(source), 
-        OMPI_FINT_2_INT(*len) * integer_type_size, 
-        FPTR_2_VOID_PTR(target), 
+    MCA_SPML_CALL(get(oshmem_ctx_default, FPTR_2_VOID_PTR(source),
+        OMPI_FINT_2_INT(*len) * integer_type_size,
+        FPTR_2_VOID_PTR(target),
         OMPI_FINT_2_INT(*pe)));
 }
- 
+
