@@ -3,10 +3,10 @@
  *                         All rights reserved.
  * Copyright (c) 2013 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
- * $HEADER$	
+ *
+ * $HEADER$
  */
 
 #include "oshmem_config.h"
@@ -18,12 +18,18 @@
 #include "ompi/datatype/ompi_datatype.h"
 #include "stdio.h"
 
+#if OSHMEM_PROFILING
+#include "oshmem/shmem/fortran/pbindings.h"
+SHMEM_GENERATE_WEAK_BINDINGS(SHMEM_IPUT32, shmem_iput32)
+#include "oshmem/shmem/fortran/profile-defines.h"
+#endif
+
 SHMEM_GENERATE_FORTRAN_BINDINGS_SUB (void,
         SHMEM_IPUT32,
         shmem_iput32_,
         shmem_iput32__,
         shmem_iput32_f,
-        (FORTRAN_POINTER_T target, FORTRAN_POINTER_T source, MPI_Fint *tst, MPI_Fint *sst, MPI_Fint *len, MPI_Fint *pe), 
+        (FORTRAN_POINTER_T target, FORTRAN_POINTER_T source, MPI_Fint *tst, MPI_Fint *sst, MPI_Fint *len, MPI_Fint *pe),
         (target,source,tst,sst,len,pe) )
 
 void shmem_iput32_f(FORTRAN_POINTER_T target, FORTRAN_POINTER_T source, MPI_Fint *tst, MPI_Fint *sst, MPI_Fint *len, MPI_Fint *pe)
@@ -34,11 +40,11 @@ void shmem_iput32_f(FORTRAN_POINTER_T target, FORTRAN_POINTER_T source, MPI_Fint
     int sst_c = OMPI_FINT_2_INT(*sst);
 
     for (i=0; i<length; i++)
-    {  
-        MCA_SPML_CALL(put((uint8_t *)FPTR_2_VOID_PTR(target) + i * tst_c * 4, 
-            4, 
-            (uint8_t *)FPTR_2_VOID_PTR(source) + i * sst_c * 4, 
+    {
+        MCA_SPML_CALL(put(oshmem_ctx_default, (uint8_t *)FPTR_2_VOID_PTR(target) + i * tst_c * 4,
+            4,
+            (uint8_t *)FPTR_2_VOID_PTR(source) + i * sst_c * 4,
             OMPI_FINT_2_INT(*pe)));
     }
 }
-    
+
