@@ -2,7 +2,12 @@
  * Copyright (c) 2013      Mellanox Technologies, Inc.
  *                         All rights reserved.
  *
- * Copyright (c) 2014 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2014-2018 Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2014-2019 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
+ * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
+ * Copyright (c) 2018      Amazon.com, Inc. or its affiliates.  All Rights reserved.
+ * Copyright (c) 2019      Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -27,41 +32,20 @@
 #endif
 
 #include MCA_timer_IMPLEMENTATION_HEADER
+#include "opal/include/opal/version.h"
 #include "opal/class/opal_value_array.h"
 #include "opal/class/opal_pointer_array.h"
 #include "opal/util/printf.h"
 #include "opal/memoryhooks/memory.h"
 #include "opal/runtime/opal_info_support.h"
 
-#if OMPI_RTE_ORTE
-#include "orte/util/show_help.h"
-#endif
-
 #include "ompi/tools/ompi_info/ompi_info.h"
-#include "ompi/include/mpi_portable_platform.h"
+#include "opal/opal_portable_platform.h"
 
 #include "oshmem/tools/oshmem_info/oshmem_info.h"
 
 
-const char *ompi_info_deprecated_value = "deprecated-ompi-info-value";
-
-static void append(char *dest, size_t max, int *first, char *src)
-{
-    size_t len;
-
-    if (NULL == src) {
-        return;
-    }
-
-    len = max - strlen(dest);
-    if (!(*first)) {
-        strncat(dest, ", ", len);
-        len = max - strlen(dest);
-    }
-    strncat(dest, src, len);
-    *first = 0;
-}
-
+const char *opal_info_deprecated_value = "deprecated-ompi-info-value";
 
 /*
  * do_config
@@ -77,47 +61,21 @@ static void append(char *dest, size_t max, int *first, char *src)
  */
 void oshmem_info_do_config(bool want_all)
 {
-    char *cxx;
-    char *fortran_mpifh;
-    char *fortran_usempi;
-    char *fortran_usempif08;
-    char *fortran_usempif08_compliance;
-    char *fortran_have_ignore_tkr;
-    char *fortran_have_f08_assumed_rank;
-    char *fortran_build_f08_subarrays;
-    char *fortran_have_optional_args;
-    char *fortran_have_bind_c;
-    char *fortran_have_private;
-    char *fortran_have_abstract;
-    char *fortran_have_asynchronous;
-    char *fortran_have_procedure;
-    char *fortran_08_using_wrappers_for_choice_buffer_functions;
-    char *java;
+    char *fortran_binding;
     char *heterogeneous;
     char *memprofile;
     char *memdebug;
     char *debug;
     char *mpi_interface_warning;
     char *cprofiling;
-    char *cxxprofiling;
-    char *fortran_mpifh_profiling;
-    char *fortran_usempi_profiling;
-    char *fortran_usempif08_profiling;
-    char *cxxexceptions;
+    char *fortran_profiling;
     char *threads;
-    char *want_libltdl;
-#if OMPI_RTE_ORTE
-    char *mpirun_prefix_by_default;
-#endif
+    char *have_dl;
     char *sparse_groups;
-    char *have_mpi_io;
     char *wtime_support;
     char *symbol_visibility;
-    char *ft_support;
-    char *crdebug_support;
     char *topology_support;
-    char *vt_support;
-    
+
     /* Do a little preprocessor trickery here to figure opal_info_out the
      * tri-state of MPI_PARAM_CHECK (which will be either 0, 1, or
      * ompi_mpi_param_check).  The preprocessor will only allow
@@ -142,170 +100,46 @@ void oshmem_info_do_config(bool want_all)
     paramcheck = "runtime";
 #endif
 
-    /* setup the strings that don't require allocations*/
-    cxx = OMPI_BUILD_CXX_BINDINGS ? "yes" : "no";
-    if (OMPI_BUILD_FORTRAN_USEMPI_BINDINGS) {
-        if (OMPI_FORTRAN_HAVE_IGNORE_TKR) {
-            fortran_usempi = "yes (full: ignore TKR)";
-        } else {
-            fortran_usempi = "yes (limited: overloading)";
-        }
-    } else {
-        fortran_usempi = "no";
-    }
-    fortran_usempif08 = OMPI_BUILD_FORTRAN_USEMPIF08_BINDINGS ? "yes" : "no";
-    fortran_have_f08_assumed_rank = OMPI_FORTRAN_HAVE_F08_ASSUMED_RANK ?
-        "yes" : "no";
-    fortran_build_f08_subarrays = OMPI_BUILD_FORTRAN_F08_SUBARRAYS ?
-        "yes" : "no";
-    fortran_have_optional_args = OMPI_FORTRAN_HAVE_OPTIONAL_ARGS ?
-        "yes" : "no";
-    fortran_have_bind_c = OMPI_FORTRAN_HAVE_BIND_C ? "yes" : "no";
-    fortran_have_private = OMPI_FORTRAN_HAVE_PRIVATE ? "yes" : "no";
-    fortran_have_abstract = OMPI_FORTRAN_HAVE_ABSTRACT ? "yes" : "no";
-    fortran_have_asynchronous = OMPI_FORTRAN_HAVE_ASYNCHRONOUS ? "yes" : "no";
-    fortran_have_procedure = OMPI_FORTRAN_HAVE_PROCEDURE ? "yes" : "no";
-    fortran_08_using_wrappers_for_choice_buffer_functions =
-        OMPI_FORTRAN_NEED_WRAPPER_ROUTINES ? "yes" : "no";
-
-    /* Build a string describing what level of compliance the mpi_f08
-       module has */
-    if (OMPI_BUILD_FORTRAN_USEMPIF08_BINDINGS) {
-
-        /* Do we have everything? */
-        if (OMPI_BUILD_FORTRAN_F08_SUBARRAYS &&
-            OMPI_FORTRAN_HAVE_PRIVATE &&
-            OMPI_FORTRAN_HAVE_ABSTRACT &&
-            OMPI_FORTRAN_HAVE_ASYNCHRONOUS &&
-            OMPI_FORTRAN_HAVE_PROCEDURE &&
-            OMPI_FORTRAN_NEED_WRAPPER_ROUTINES) {
-            fortran_usempif08_compliance = strdup("The mpi_f08 module is available, and is fully compliant.  w00t!");
-        } else {
-            char f08[1024];
-            int first = 1;
-            snprintf(f08, sizeof(f08),
-                     "The mpi_f08 module is available, but due to limitations in the %s compiler, does not support the following: ",
-                     OMPI_FC);
-            if (!OMPI_BUILD_FORTRAN_F08_SUBARRAYS) {
-                append(f08, sizeof(f08), &first, "array subsections");
-            }
-            if (!OMPI_FORTRAN_HAVE_PRIVATE) {
-                append(f08, sizeof(f08), &first, "private MPI_Status members");
-            }
-            if (!OMPI_FORTRAN_HAVE_ABSTRACT) {
-                append(f08, sizeof(f08), &first, "ABSTRACT INTERFACE function pointers");
-            }
-            if (!OMPI_FORTRAN_HAVE_ASYNCHRONOUS) {
-                append(f08, sizeof(f08), &first, "Fortran '08-specified ASYNCHRONOUS behavior");
-            }
-            if (!OMPI_FORTRAN_HAVE_PROCEDURE) {
-                append(f08, sizeof(f08), &first, "PROCEDUREs");
-            }
-            if (OMPI_FORTRAN_NEED_WRAPPER_ROUTINES) {
-                append(f08, sizeof(f08), &first, "direct passthru (where possible) to underlying Open MPI's C functionality");
-            }
-            fortran_usempif08_compliance = strdup(f08);
-        }
-    } else {
-        fortran_usempif08_compliance = strdup("The mpi_f08 module was not built");
-    }
-
-    java = OMPI_WANT_JAVA_BINDINGS ? "yes" : "no";
     heterogeneous = OPAL_ENABLE_HETEROGENEOUS_SUPPORT ? "yes" : "no";
     memprofile = OPAL_ENABLE_MEM_PROFILE ? "yes" : "no";
     memdebug = OPAL_ENABLE_MEM_DEBUG ? "yes" : "no";
     debug = OPAL_ENABLE_DEBUG ? "yes" : "no";
     mpi_interface_warning = OMPI_WANT_MPI_INTERFACE_WARNING ? "yes" : "no";
-    cprofiling = OMPI_ENABLE_MPI_PROFILING ? "yes" : "no";
-    cxxprofiling = (OMPI_BUILD_CXX_BINDINGS && OMPI_ENABLE_MPI_PROFILING) ? "yes" : "no";
-    cxxexceptions = (OMPI_BUILD_CXX_BINDINGS && OMPI_HAVE_CXX_EXCEPTION_SUPPORT) ? "yes" : "no";
-    fortran_mpifh_profiling = (OMPI_ENABLE_MPI_PROFILING && OMPI_BUILD_FORTRAN_MPIFH_BINDINGS) ? "yes" : "no";
-    fortran_usempi_profiling = (OMPI_ENABLE_MPI_PROFILING && OMPI_BUILD_FORTRAN_USEMPI_BINDINGS) ? "yes" : "no";
-    fortran_usempif08_profiling = (OMPI_ENABLE_MPI_PROFILING && OMPI_BUILD_FORTRAN_USEMPIF08_BINDINGS) ? "yes" : "no";
-    want_libltdl = OPAL_WANT_LIBLTDL ? "yes" : "no";
-#if OMPI_RTE_ORTE
-    mpirun_prefix_by_default = ORTE_WANT_ORTERUN_PREFIX_BY_DEFAULT ? "yes" : "no";
-#endif
+    cprofiling = "yes";
+    fortran_profiling = (OMPI_BUILD_FORTRAN_BINDINGS >= OMPI_FORTRAN_MPIFH_BINDINGS) ? "yes" : "no";
+    have_dl = OPAL_HAVE_DL_SUPPORT ? "yes" : "no";
     sparse_groups = OMPI_GROUP_SPARSE ? "yes" : "no";
-    have_mpi_io = OMPI_PROVIDE_MPI_FILE_INTERFACE ? "yes" : "no";
     wtime_support = OPAL_TIMER_USEC_NATIVE ? "native" : "gettimeofday";
     symbol_visibility = OPAL_C_HAVE_VISIBILITY ? "yes" : "no";
-    topology_support = OPAL_HAVE_HWLOC ? "yes" : "no";
-    vt_support = OMPI_ENABLE_CONTRIB_vt ? "yes" : "no";
+    topology_support = "yes";
 
     /* setup strings that require allocation */
-    if (OMPI_BUILD_FORTRAN_MPIFH_BINDINGS) {
-        (void)asprintf(&fortran_mpifh, "yes (%s)",
+    if (OMPI_BUILD_FORTRAN_BINDINGS >= OMPI_FORTRAN_MPIFH_BINDINGS) {
+        (void)opal_asprintf(&fortran_binding, "yes (%s)",
                        (OPAL_HAVE_WEAK_SYMBOLS ? "all" :
                         (OMPI_FORTRAN_CAPS ? "caps" :
                          (OMPI_FORTRAN_PLAIN ? "lower case" :
                           (OMPI_FORTRAN_SINGLE_UNDERSCORE ? "single underscore" : "double underscore")))));
     } else {
-        fortran_mpifh = strdup("no");
+        fortran_binding = strdup("no");
     }
 
-    if (OMPI_FORTRAN_HAVE_IGNORE_TKR) {
-        /* OMPI_FORTRAN_IGNORE_TKR_PREDECL is already in quotes; it
-           didn't work consistently to put it in _STRINGIFY because
-           sometimes the compiler would actually interpret the pragma
-           in there before stringify-ing it. */
-        (void)asprintf(&fortran_have_ignore_tkr, "yes (%s)",
-                       OMPI_FORTRAN_IGNORE_TKR_PREDECL);
-    } else {
-        fortran_have_ignore_tkr = strdup("no");
-    }
-
-    if (OPAL_HAVE_POSIX_THREADS) {        /* should just test OPAL_HAVE_THREADS */
-#if OMPI_RTE_ORTE
-        (void)asprintf(&threads, "%s (MPI_THREAD_MULTIPLE: %s, OPAL support: %s, OMPI progress: %s, ORTE progress: yes, Event lib: yes)",
-                       (OPAL_HAVE_POSIX_THREADS ? "posix" : "type unknown"), /* "type unknown" can presumably never happen */
-                       OMPI_ENABLE_THREAD_MULTIPLE ? "yes" : "no",
-                       OPAL_ENABLE_MULTI_THREADS ? "yes" : "no",
-                       OMPI_ENABLE_PROGRESS_THREADS ? "yes" : "no");
-#else
-        (void)asprintf(&threads, "%s (MPI_THREAD_MULTIPLE: %s, OPAL support: %s, OMPI progress: %s, Event lib: yes)",
-                       (OPAL_HAVE_POSIX_THREADS ? "posix" : "type unknown"), /* "type unknown" can presumably never happen */
-                       OMPI_ENABLE_THREAD_MULTIPLE ? "yes" : "no",
-                       OPAL_ENABLE_MULTI_THREADS ? "yes" : "no",
-                       OMPI_ENABLE_PROGRESS_THREADS ? "yes" : "no");
-#endif
-    } else {
-        threads = strdup("no");
-    }
-
-    (void)asprintf(&ft_support, "%s (checkpoint thread: %s)",
-                   OPAL_ENABLE_FT ? "yes" : "no", OPAL_ENABLE_FT_THREAD ? "yes" : "no");
-
-    (void)asprintf(&crdebug_support, "%s",
-                   OPAL_ENABLE_CRDEBUG ? "yes" : "no");
+    (void)opal_asprintf(&threads, "%s (MPI_THREAD_MULTIPLE: yes, OPAL support: yes, OMPI progress: %s, Event lib: yes)",
+                   "posix", OPAL_ENABLE_PROGRESS_THREADS ? "yes" : "no");
 
     /* output values */
     opal_info_out("Configured by", "config:user", OPAL_CONFIGURE_USER);
     opal_info_out("Configured on", "config:timestamp", OPAL_CONFIGURE_DATE);
     opal_info_out("Configure host", "config:host", OPAL_CONFIGURE_HOST);
+    opal_info_out("Configure command line", "config:cli", OPAL_CONFIGURE_CLI);
 
     opal_info_out("Built by", "build:user", OMPI_BUILD_USER);
     opal_info_out("Built on", "build:timestamp", OMPI_BUILD_DATE);
     opal_info_out("Built host", "build:host", OMPI_BUILD_HOST);
 
     opal_info_out("C bindings", "bindings:c", "yes");
-    opal_info_out("C++ bindings", "bindings:cxx", cxx);
-    opal_info_out("Fort mpif.h", "bindings:mpif.h", fortran_mpifh);
-    free(fortran_mpifh);
-    opal_info_out("Fort use mpi", "bindings:use_mpi",
-                  fortran_usempi);
-    opal_info_out("Fort use mpi size", "bindings:use_mpi:size",
-                  ompi_info_deprecated_value);
-    opal_info_out("Fort use mpi_f08", "bindings:use_mpi_f08",
-                  fortran_usempif08);
-    opal_info_out("Fort mpi_f08 compliance", "bindings:use_mpi_f08:compliance",
-                  fortran_usempif08_compliance);
-    if (NULL != fortran_usempif08_compliance) {
-        free(fortran_usempif08_compliance);
-    }
-    opal_info_out("Fort mpi_f08 subarrays", "bindings:use_mpi_f08:subarrays-supported",
-                  fortran_build_f08_subarrays);
-    opal_info_out("Java bindings", "bindings:java", java);
+    opal_info_out("Fort shmem.fh", "bindings:fortran", fortran_binding);
+    free(fortran_binding);
 
     opal_info_out("Wrapper compiler rpath", "compiler:all:rpath",
                   WRAPPER_RPATH_SUPPORT);
@@ -313,9 +147,9 @@ void oshmem_info_do_config(bool want_all)
     opal_info_out("C compiler absolute", "compiler:c:absolute",
                   OPAL_CC_ABSOLUTE);
     opal_info_out("C compiler family name", "compiler:c:familyname",
-                  _STRINGIFY(OPAL_BUILD_PLATFORM_COMPILER_FAMILYNAME));
+                  PLATFORM_STRINGIFY(PLATFORM_COMPILER_FAMILYNAME));
     opal_info_out("C compiler version", "compiler:c:version",
-                  _STRINGIFY(OPAL_BUILD_PLATFORM_COMPILER_VERSION_STR));
+                  PLATFORM_COMPILER_VERSION_STR);
 
     if (want_all) {
         opal_info_out_int("C char size", "compiler:c:sizeof:char", sizeof(char));
@@ -334,14 +168,7 @@ void oshmem_info_do_config(bool want_all)
         opal_info_out_int("C double size", "compiler:c:sizeof:double", sizeof(double));
         opal_info_out_int("C pointer size", "compiler:c:sizeof:pointer", sizeof(void *));
         opal_info_out_int("C char align", "compiler:c:align:char", OPAL_ALIGNMENT_CHAR);
-#if OMPI_BUILD_CXX_BINDINGS
-        /* JMS: See above for note about C++ bool size.  We don't have
-           the bool alignment the way configure currently runs -- need
-           to clean this up when we update for MPI-2.2. */
-        opal_info_out_int("C bool align", "compiler:c:align:bool", OPAL_ALIGNMENT_CXX_BOOL);
-#else
         opal_info_out("C bool align", "compiler:c:align:bool", "skipped");
-#endif
         opal_info_out_int("C int align", "compiler:c:align:int", OPAL_ALIGNMENT_INT);
         opal_info_out_int("C float align", "compiler:c:align:float", OPAL_ALIGNMENT_FLOAT);
         opal_info_out_int("C double align", "compiler:c:align:double", OPAL_ALIGNMENT_DOUBLE);
@@ -352,33 +179,6 @@ void oshmem_info_do_config(bool want_all)
     opal_info_out("Fort compiler", "compiler:fortran:command", OMPI_FC);
     opal_info_out("Fort compiler abs", "compiler:fortran:absolute",
                   OMPI_FC_ABSOLUTE);
-    opal_info_out("Fort ignore TKR", "compiler:fortran:ignore_tkr",
-                  fortran_have_ignore_tkr);
-    free(fortran_have_ignore_tkr);
-    opal_info_out("Fort 08 assumed shape",
-                  "compiler:fortran:f08_assumed_rank",
-                  fortran_have_f08_assumed_rank);
-    opal_info_out("Fort optional args",
-                  "compiler:fortran:optional_arguments",
-                  fortran_have_optional_args);
-    opal_info_out("Fort BIND(C)",
-                  "compiler:fortran:bind_c",
-                  fortran_have_bind_c);
-    opal_info_out("Fort PRIVATE",
-                  "compiler:fortran:private",
-                  fortran_have_private);
-    opal_info_out("Fort ABSTRACT",
-                  "compiler:fortran:abstract",
-                  fortran_have_abstract);
-    opal_info_out("Fort ASYNCHRONOUS",
-                  "compiler:fortran:asynchronous",
-                  fortran_have_asynchronous);
-    opal_info_out("Fort PROCEDURE",
-                  "compiler:fortran:procedure",
-                  fortran_have_procedure);
-    opal_info_out("Fort f08 using wrappers",
-                  "compiler:fortran:08_wrappers",
-                  fortran_08_using_wrappers_for_choice_buffer_functions);
 
     if (want_all) {
 
@@ -395,9 +195,7 @@ void oshmem_info_do_config(bool want_all)
 
         /* May or may not have the other Fortran sizes */
 
-        if (OMPI_BUILD_FORTRAN_MPIFH_BINDINGS ||
-            OMPI_BUILD_FORTRAN_USEMPI_BINDINGS ||
-            OMPI_BUILD_FORTRAN_USEMPIF08_BINDINGS) {
+        if (OMPI_BUILD_FORTRAN_BINDINGS >= OMPI_FORTRAN_MPIFH_BINDINGS) {
             opal_info_out("Fort have integer1", "compiler:fortran:have:integer1",
                           OMPI_HAVE_FORTRAN_INTEGER1 ? "yes" : "no");
             opal_info_out("Fort have integer2", "compiler:fortran:have:integer2",
@@ -516,16 +314,9 @@ void oshmem_info_do_config(bool want_all)
     }
 
     opal_info_out("C profiling", "option:profiling:c", cprofiling);
-    opal_info_out("C++ profiling", "option:profiling:cxx", cxxprofiling);
-    opal_info_out("Fort mpif.h profiling", "option:profiling:mpif.h",
-                  fortran_mpifh_profiling);
-    opal_info_out("Fort use mpi profiling", "option:profiling:use_mpi",
-                  fortran_usempi_profiling);
-    opal_info_out("Fort use mpi_f08 prof",
-                  "option:profiling:use_mpi_f08",
-                  fortran_usempif08_profiling);
+    opal_info_out("Fort shmem.fh profiling", "option:profiling:shmem.fh",
+                  fortran_profiling);
 
-    opal_info_out("C++ exceptions", "option:cxx_exceptions", cxxexceptions);
     opal_info_out("Thread support", "option:threads", threads);
     free(threads);
     opal_info_out("Sparse Groups", "option:sparse:groups", sparse_groups);
@@ -543,16 +334,16 @@ void oshmem_info_do_config(bool want_all)
         opal_info_out("Build LDFLAGS", "option:build:ldflags", OMPI_BUILD_LDFLAGS);
         opal_info_out("Build LIBS", "option:build:libs", OMPI_BUILD_LIBS);
 
-        opal_info_out("Wrapper extra CFLAGS", "option:wrapper:extra_cflags",
-                      WRAPPER_EXTRA_CFLAGS);
-        opal_info_out("Wrapper extra CXXFLAGS", "option:wrapper:extra_cxxflags",
-                      WRAPPER_EXTRA_CXXFLAGS);
-        opal_info_out("Wrapper extra FCFLAGS", "option:wrapper:extra_fcflags",
-                      WRAPPER_EXTRA_FCFLAGS);
-        opal_info_out("Wrapper extra LDFLAGS", "option:wrapper:extra_ldflags",
-                      WRAPPER_EXTRA_LDFLAGS);
-        opal_info_out("Wrapper extra LIBS", "option:wrapper:extra_libs",
-                      WRAPPER_EXTRA_LIBS);
+        opal_info_out("Wrapper CFLAGS", "option:wrapper:cflags",
+                      OSHMEM_WRAPPER_CFLAGS);
+        opal_info_out("Wrapper CXXFLAGS", "option:wrapper:cxxflags",
+                      OSHMEM_WRAPPER_CXXFLAGS);
+        opal_info_out("Wrapper FCFLAGS", "option:wrapper:fcflags",
+                      OSHMEM_WRAPPER_FCFLAGS);
+        opal_info_out("Wrapper LDFLAGS", "option:wrapper:ldflags",
+                      OSHMEM_WRAPPER_LDFLAGS);
+        opal_info_out("Wrapper LIBS", "option:wrapper:libs",
+                      OSHMEM_WRAPPER_LIBS);
     }
 
     opal_info_out("Internal debug support", "option:debug", debug);
@@ -560,13 +351,8 @@ void oshmem_info_do_config(bool want_all)
     opal_info_out("MPI parameter check", "option:mpi-param-check", paramcheck);
     opal_info_out("Memory profiling support", "option:mem-profile", memprofile);
     opal_info_out("Memory debugging support", "option:mem-debug", memdebug);
-    opal_info_out("libltdl support", "option:dlopen", want_libltdl);
+    opal_info_out("dl support", "option:dlopen", have_dl);
     opal_info_out("Heterogeneous support", "options:heterogeneous", heterogeneous);
-#if OMPI_RTE_ORTE
-    opal_info_out("mpirun default --prefix", "mpirun:prefix_by_default",
-                  mpirun_prefix_by_default);
-#endif
-    opal_info_out("MPI I/O support", "options:mpi-io", have_mpi_io);
     opal_info_out("MPI_WTIME support", "options:mpi-wtime", wtime_support);
     opal_info_out("Symbol vis. support", "options:visibility", symbol_visibility);
     opal_info_out("Host topology support", "options:host-topology",
@@ -574,14 +360,6 @@ void oshmem_info_do_config(bool want_all)
 
     opal_info_out("MPI extensions", "options:mpi_ext", OMPI_MPIEXT_COMPONENTS);
 
-    opal_info_out("FT Checkpoint support", "options:ft_support", ft_support);
-    free(ft_support);
-
-    opal_info_out("C/R Enabled Debugging", "options:crdebug_support", crdebug_support);
-    free(crdebug_support);
-
-    opal_info_out("VampirTrace support", "options:vt", vt_support);
-
     opal_info_out_int("MPI_MAX_PROCESSOR_NAME", "options:mpi-max-processor-name",
                   MPI_MAX_PROCESSOR_NAME);
     opal_info_out_int("MPI_MAX_ERROR_STRING",   "options:mpi-max-error-string",
@@ -594,14 +372,9 @@ void oshmem_info_do_config(bool want_all)
                   MPI_MAX_INFO_VAL);
     opal_info_out_int("MPI_MAX_PORT_NAME",      "options:mpi-max-port-name",
                   MPI_MAX_PORT_NAME);
-#if OMPI_PROVIDE_MPI_FILE_INTERFACE
     opal_info_out_int("MPI_MAX_DATAREP_STRING", "options:mpi-max-datarep-string",
                   MPI_MAX_DATAREP_STRING);
-#else
-    opal_info_out("MPI_MAX_DATAREP_STRING", "options:mpi-max-datarep-string",
-                  "IO interface not provided");
-#endif
-    
+
     /* This block displays all the options with which the current
      * installation of oshmem was configured. */
     {
@@ -609,7 +382,7 @@ void oshmem_info_do_config(bool want_all)
         char *oshmem_compat = OSHMEM_SPEC_COMPAT ? "yes" : "no";
         char *oshmem_param_check = OSHMEM_PARAM_CHECK ? "yes" : "no";
         char *oshmem_profiling = OSHMEM_PROFILING ? "yes" : "no";
-        
+
         opal_info_out("OSHMEM C bindings", "oshmem:bindings:c", "yes");
         opal_info_out("OSHMEM Fortran bindings", "oshmem:bindings:fort", oshmem_fortran);
         opal_info_out("OSHMEM SGI/Quadrics mode", "oshmem:options:spec_compat", oshmem_compat);
